#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "err.h"
#include "str.h"
#include "cfg.h"

#define CHAR_SEP  	':'

cfg_t *first_cfg = NULL;
cfg_t *last_cfg = NULL;

int cfg_add (char *name, char *value) {
	cfg_t *new_cfg;

	if (!name || !value) {
		err_report (E_PARAM, "cfg_add: no name or value given");
		return 0;
	}

	new_cfg = (cfg_t *) malloc (sizeof (cfg_t));

	if (!last_cfg) 
		first_cfg = new_cfg;
	else
		last_cfg->next = new_cfg;

	new_cfg->name = strdup (name);
	new_cfg->value = strdup (value);
	new_cfg->next = NULL;

	last_cfg = new_cfg;

	return 1;
}

int cfg_expand (char *name, char *value) {
	cfg_t *search_cfg;

	if (!name || !value) {
		err_report (E_PARAM, "cfg_expand: no name or value given");
		return 0;
	}

	for (search_cfg = first_cfg;
	     search_cfg;
	     search_cfg = search_cfg->next) 
		if (!strcasecmp (search_cfg->name, name))  {
			search_cfg->value = (char *) realloc (search_cfg->value, strlen (search_cfg->value) + strlen (value) + 1);
			strcat (search_cfg->value, " ");
			strcat (search_cfg->value, value);

			return 1;
		}

	return 0;
}

int cfg_read (char *file_name) {
	FILE *f;
	char *name;
	char *value;
	char *str;
	int i, join_lines = 0;

	str = (char *) alloca (MAX_SIZE);
	name = (char *) alloca (MAX_SIZE);

	if (!file_name) {
		err_report (E_PARAM, "cfg_read: filename not set");
		return 0;
	}

	if (!(f = fopen (file_name, "r"))) {
		err_report (E_FILE, "cfg_read: could not open %s", file_name);
		return 0;
	}

	while (fgets (str, MAX_SIZE - 1, f)) {
		str = str_chop (str, 0);

		if (str[0] == '#') {
			join_lines = 0;
			continue;
		}

		if (join_lines) {
			if (str[strlen(str) - 1] == '\\') {
				str[strlen(str) - 1] = 0;
				join_lines = 1;
			} else
				join_lines = 0;

			cfg_expand (name, str);
			continue;
		}

		if (strchr (str, CHAR_SEP)) {
			for (i = 0; i < strlen (str); i++) {
				name[i] = str[i];
				if (str[i] == CHAR_SEP) break;
			}

			for (i = i - 1; i > 0; i--)
				if (str[i] != 32) break;
			name[i + 1] = 0;

			for (i = i + 1; i < strlen (str); i++)
				if ((str[i] != CHAR_SEP) && (str[i] != ' ') && (str[i] != '\t'))
					break;

			value = (char *) alloca (strlen (str) - i + 1);
			strcpy (value, &str[i]);
			if (value[strlen(value) - 1] == '\\') {
				value[strlen(value) - 1] = 0;
				join_lines = 1;
			}

			cfg_add (name, value);

			str[0] = 0;
		}
	}

	fclose (f);
	return 1;
}

char *cfg_retr (char *name) {
	cfg_t *search_cfg;

	for (search_cfg = first_cfg;
	     search_cfg;
	     search_cfg = search_cfg->next) 
		if (!strcasecmp (search_cfg->name, name)) 
			return search_cfg->value;

	err_report (E_UNKNOWN, "cfg_retr: '%s' is not set in configuration!", name);
	return strdup ("");
}

char *cfg_get (char *name) {
	cfg_t *search_cfg;

	for (search_cfg = first_cfg;
	     search_cfg;
	     search_cfg = search_cfg->next)
		if (!strcasecmp (search_cfg->name, name))
			return search_cfg->value;

	printf ("** FATAL: Variable '%s' is not set in configuration, cannot continue\n", name);
	fflush (stdout);

	exit (0);	
}

cfg_t *cfg_first() {
	return first_cfg;
}

int cfg_write (char *file_name, char *pre) {
	FILE *f;
	cfg_t *cfg;
	int i;

	if (!file_name) {
		err_report (E_PARAM, "cfg_write: filename not set");
		return 0;
	}

	if (!(f = fopen (file_name, "w"))) {
		err_report (E_FILE, "cfg_write: could not open %s", file_name);
		return 0;
	}

	if (pre && strlen (pre)) 
		fprintf (f, "%s\n", pre);

	for (cfg = first_cfg; cfg; cfg=cfg->next) {
		fprintf (f, "%s:\t", cfg->name);
		i = 0;
		while (cfg->value[i]) {
			if (cfg->value[i + 1] && (cfg->value[i] == ' ') &&
			    (cfg->value[i + 1] == ' ')) {
				fprintf (f, " \\\n\t\t\t");
				while (cfg->value[i] == ' ')
					i++;
			}

			if (cfg->value[i]) {
				fputc (cfg->value[i], f);
				i++;
			}
		}
		fputc ('\n', f);
	}

	fclose (f);

	return 1;
}

int cfg_free () {
	cfg_t *cfg1, *cfg2;

	cfg1 = first_cfg;
	while (cfg1) {
		cfg2 = cfg1->next;
		free (cfg1->name);
		free (cfg1->value);
		free (cfg1);
		cfg1 = cfg2;
	}

	first_cfg = NULL;
	last_cfg = NULL;

	return 1;
}
