#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include "err.h"
#include "cfg.h"
#include "htdb.h"

int main (int argc, char *argv[]) {
	char *db_user, *db_pass, *db_name, *cron_user; 
	int htdb_id;
	struct stat stat_buf;
	htdb_t htdb_stats;

	printf ("ht://Dig file <-> database synchronizer\n\n");

	if (argc < 3) {
		printf ("usage: %s <htsync_cfg> <installation_cfg>\n", argv[0]);	
		return 1;
	}

	if (!cfg_read (argv[1])) {
		printf ("( fail ) %s\n", err_get());
		return 2;
	}

	db_user = strdup (cfg_get("db_user"));
	db_pass = strdup (cfg_get("db_pass"));
	db_name = strdup (cfg_get("db_name"));
	cron_user = strdup (cfg_get("cron_user"));

	cfg_free();

	printf ("1. Initializing configfile and database\n\n");
	fflush (stdout);

	if (!cfg_read (argv[2])) {
		printf ("( fail ) %s\n", err_get());
		return 3;
	}

	if (!(htdb_id = htdb_init (db_name, db_user, db_pass, argv[2]))) {
		printf ("( fail ) %s\n", err_get());
		return 4;
	}

	printf ("(  ok  ) database handle is %i\n\n", htdb_id);


	/* sync the configuration file with the database */

	printf ("2. Syncing configurationfile with database (two-way)\n\n");
	fflush (stdout);

	stat (argv[2], &stat_buf);

	htdb_stats = htdb_configsync (htdb_id, stat_buf.st_mtime);

	if (err_getno() != E_NONE) {
		printf ("( fail ) %s\n", err_get());
		return 5;
	}

	if (!cfg_write (argv[2], "# This configurationfile is maintained by HTSync.\n#\n# Before making changes, run HTSync to make sure this file\n# contains the latest database changes.\n#\n# After making changes, run HTSync again to update the database\n# with the latest changes in this file.\n#")) {
		printf ("( fail ) %s\n", err_get());
		return 6;
	}

	printf ("(  ok  ) file '%s' -> database '%s' : %i items\n", argv[2], db_name, htdb_stats.file2db_count);
	printf ("(  ok  ) file '%s' <- database '%s' : %i items\n\n", argv[2], db_name, htdb_stats.db2file_count);


	/* sync index database properties with the database */

	printf ("3. Updating database with file index properties\n\n");
	fflush (stdout);

	htdb_stats = htdb_indexsync (htdb_id);

	if (err_getno() == E_NONE) 
		printf ("(  ok  ) index size = %iK, last updated at %s\n", (int)htdb_stats.index_size, ctime(&htdb_stats.index_updated));
	else 	
		printf ("( fail ) %s\n\n", err_get());


	/* sync crontab entry with the database */

	printf ("4. Updating database with crontab entry\n\n");
	fflush (stdout);

	htdb_stats = htdb_cronsync (htdb_id, cron_user, argv[2]);

	if (err_getno() == E_NONE)
		printf ("(  ok  ) crontab %s database\n\n", htdb_stats.cron_res ? "->" : "<-");
	else
		printf ("( fail ) %s\n\n", err_get());


	printf ("Done..\n\n");
	return 0; 
}
